searchData={"items":[{"type":"module","title":"Assertions","doc":"Helpful assertions with great error messages to help you write better tests.","ref":"Assertions.html"},{"type":"macro","title":"Assertions.assert!/1","doc":"Asserts that the return value of the given expression is `true`.\n\nThis is different than the normal behavior of `assert` since that will pass\nfor any value that is \"truthy\" (anything other than `false` or `nil`). This is\na stricter assertion, only passing if the value is `true`. This is very\nhelpful for testing values that are expected to only be booleans.\n\nThis will also check specifically for `nil` values when using `>`, `<`, `>=`\nor `<=` since those frequently have unintended behavior.\n\n    iex> assert!(:a == :a)\n    true\n    iex> assert!(10 > 5)\n    true\n    iex> map = %{key: true}\n    iex> assert!(map.key)\n    true","ref":"Assertions.html#assert!/1"},{"type":"macro","title":"Assertions.assert_all_have_value/3","doc":"Asserts that all maps, structs or keyword lists in `list` have the same\n`value` for `key`.\n\n    iex> assert_all_have_value([%{key: :value}, %{key: :value, other: :key}], :key, :value)\n    true\n\n    iex> assert_all_have_value([[key: :value], [key: :value, other: :key]], :key, :value)\n    true\n\n    iex> assert_all_have_value([[key: :value], %{key: :value, other: :key}], :key, :value)\n    true","ref":"Assertions.html#assert_all_have_value/3"},{"type":"macro","title":"Assertions.assert_async/2","doc":"Asserts that some condition succeeds within a given timeout (in milliseconds)\nand sleeps for a given time between checks of the given condition (in\nmilliseconds).\n\nThis is helpful for testing that asynchronous operations have succeeded within\na certain timeframe. This method of testing asynchronous operations is less\nreliable than other methods, but it can often be more useful at an integration\nlevel.\n\n    iex> Process.send_after(self(), :hello, 50)\n    iex> assert_async do\n    iex>   assert_received :hello\n    iex> end\n    true\n\n    iex> Process.send_after(self(), :hello, 50)\n    iex> assert_async(timeout: 75, sleep_time: 40) do\n    iex>   assert_received :hello\n    iex> end\n    true\n\n    iex> Process.send_after(self(), :hello, 50)\n    iex> try do\n    iex>   assert_async(timeout: 4, sleep_time: 2) do\n    iex>     assert_received :hello\n    iex>   end\n    iex> rescue\n    iex>   _ -> :failed\n    iex> end\n    :failed","ref":"Assertions.html#assert_async/2"},{"type":"macro","title":"Assertions.assert_changes_file/3","doc":"Asserts that the file at `path` is changed to match `comparison` after\nexecuting the given `expression`.\n\nIf the file matches `comparison` before executing `expr`, this assertion will\nfail. The file does not have to exist before executing `expr` in order for\nthis assertion to pass.\n\n    iex> path = Path.expand(\"../tmp/file.txt\", __DIR__)\n    iex> result = assert_changes_file(path, \"hi\") do\n    iex>   File.mkdir_p!(Path.dirname(path))\n    iex>   File.write(path, \"hi\")\n    iex> end\n    iex> File.rm_rf!(Path.dirname(path))\n    iex> result\n    true","ref":"Assertions.html#assert_changes_file/3"},{"type":"macro","title":"Assertions.assert_creates_file/2","doc":"Asserts that the file at `path` is created after executing the given\n`expression`.\n\n    iex> path = Path.expand(\"../tmp/file.txt\", __DIR__)\n    iex> File.mkdir_p!(Path.dirname(path))\n    iex> result = assert_creates_file path do\n    iex>   File.write(path, \"hi\")\n    iex> end\n    iex> File.rm_rf!(Path.dirname(path))\n    iex> result\n    true","ref":"Assertions.html#assert_creates_file/2"},{"type":"macro","title":"Assertions.assert_deletes_file/2","doc":"Asserts that the file at `path` is deleted after executing the given\n`expression`.\n\n    iex> path = Path.expand(\"../tmp/file.txt\", __DIR__)\n    iex> File.mkdir_p!(Path.dirname(path))\n    iex> File.write(path, \"hi\")\n    iex> assert_deletes_file path do\n    iex>   File.rm_rf!(Path.dirname(path))\n    iex> end\n    true","ref":"Assertions.html#assert_deletes_file/2"},{"type":"macro","title":"Assertions.assert_lists_equal/2","doc":"Asserts that two lists contain the same elements without asserting they are\nin the same order.\n\n    iex> assert_lists_equal([1, 2, 3], [1, 3, 2])\n    true","ref":"Assertions.html#assert_lists_equal/2"},{"type":"macro","title":"Assertions.assert_lists_equal/3","doc":"Asserts that two lists contain the same elements without asserting they are\nin the same order.\n\nThe given comparison function determines if the two lists are considered\nequal.\n\n    iex> assert_lists_equal([\"dog\"], [\"cat\"], &(is_binary(&1) and is_binary(&2)))\n    true","ref":"Assertions.html#assert_lists_equal/3"},{"type":"macro","title":"Assertions.assert_map_in_list/3","doc":"Asserts that a `map` is in the given `list`.\n\nThis is either done by passing a list of `keys`, and the values at those keys\nwill be compared to determine if the map is in the list.\n\n    iex> map = %{first: :first, second: :second}\n    iex> list = [%{first: :first, second: :second, third: :third}]\n    iex> keys = [:first, :second]\n    iex> assert_map_in_list(map, list, keys)\n    true\n\nOr this is done by passing a comparison function that determines if the map\nis in the list.\n\nIf using a comparison function, the `map` is the first argument to that\nfunction, and the elements in the list are the second argument.\n\n    iex> map = %{first: :first, second: :second}\n    iex> list = [%{\"first\" => :first, \"second\" => :second, \"third\" => :third}]\n    iex> comparison = &(&1.first == &2[\"first\"] and &1.second == &2[\"second\"])\n    iex> assert_map_in_list(map, list, comparison)\n    true","ref":"Assertions.html#assert_map_in_list/3"},{"type":"macro","title":"Assertions.assert_maps_equal/3","doc":"Asserts that two maps are equal.\n\nEquality can be determined in two ways. First, by passing a list of keys. The\nvalues at these keys will be used to determine if the maps are equal.\n\n    iex> left = %{first: :first, second: :second, third: :third}\n    iex> right = %{first: :first, second: :second, third: :fourth}\n    iex> keys = [:first, :second]\n    iex> assert_maps_equal(left, right, keys)\n    true\n\nThe second is to pass a comparison function that returns a boolean that\ndetermines if the maps are equal. When using a comparison function, the first\nargument to the function is the `left` map and the second argument is the\n`right` map.\n\n    iex> left = %{first: :first, second: :second, third: :third}\n    iex> right = %{\"first\" => :first, \"second\" => :second, \"third\" => :fourth}\n    iex> comparison = &(&1.first == &2[\"first\"] and &1.second == &2[\"second\"])\n    iex> assert_maps_equal(left, right, comparison)\n    true","ref":"Assertions.html#assert_maps_equal/3"},{"type":"function","title":"Assertions.assert_raise/1","doc":"Asserts that a function should raise an exception, but without forcing the user to specify which\nexception should be raised. This is essentially a less-strict version of `assert_raise/2`.\n\n    iex> assert_raise(fn -> String.to_existing_atom(\"asleimflisesliseli\") end)\n    true","ref":"Assertions.html#assert_raise/1"},{"type":"macro","title":"Assertions.assert_receive_only/2","doc":"Tests that a message matching the given `pattern`, and only that message, is\nreceived before the given `timeout`, specified in milliseconds.\n\nThe optional second argument is a timeout for the `receive` to wait for the\nexpected message, and defaults to 100ms.","ref":"Assertions.html#assert_receive_only/2"},{"type":"macro","title":"Examples - Assertions.assert_receive_only/2","doc":"iex> send(self(), :hello)\n    iex> assert_receive_only(:hello)\n    true\n\n    iex> send(self(), [:hello])\n    iex> assert_receive_only([_])\n    true\n\n    iex> a = :hello\n    iex> send(self(), :hello)\n    iex> assert_receive_only(^a)\n    true\n\n    iex> send(self(), :hello)\n    iex> assert_receive_only(a when is_atom(a))\n    iex> a\n    :hello\n\n    iex> send(self(), %{key: :value})\n    iex> assert_receive_only(%{key: value} when is_atom(value))\n    iex> value\n    :value\n\nIf a message is received after the assertion has matched a message to the\ngiven pattern, but the second message is received before the timeout, that\nsecond message is ignored and the assertion returns `true`.\n\nThis assertion only tests that the message that matches the given pattern was\nthe first message in the process inbox, and that nothing was sent between the\nsending the message that matches the pattern and when `assert_receive_only/2`\nwas called.\n\n    iex> Process.send_after(self(), :hello, 20)\n    iex> Process.send_after(self(), :hello_again, 50)\n    iex> assert_receive_only(:hello, 100)\n    true","ref":"Assertions.html#assert_receive_only/2-examples"},{"type":"macro","title":"Assertions.assert_struct_in_list/3","doc":"Asserts that the `struct` is present in the `list`.\n\nThere are two ways to make this comparison. First is to pass a list of keys\nto use to compare the `struct` to the structs in the `list`.\n\n    iex> now = DateTime.utc_now()\n    iex> list = [DateTime.utc_now(), Date.utc_today()]\n    iex> keys = [:year, :month, :day]\n    iex> assert_struct_in_list(now, list, keys)\n    true\n\nThe second way to use this assertion is to pass a comparison function.\n\nWhen using a comparison function, the `struct` is the first argument to that\nfunction and the elements in the `list` will be the second argument.\n\n    iex> now = DateTime.utc_now()\n    iex> list = [DateTime.utc_now(), Date.utc_today()]\n    iex> assert_struct_in_list(now, list, &(&1.year == &2.year))\n    true","ref":"Assertions.html#assert_struct_in_list/3"},{"type":"macro","title":"Assertions.assert_structs_equal/3","doc":"Asserts that two structs are equal.\n\nEquality can be determined in two ways. First, by passing a list of keys. The\nvalues at these keys and the type of the structs will be used to determine if\nthe structs are equal.\n\n    iex> left = DateTime.utc_now()\n    iex> right = DateTime.utc_now()\n    iex> keys = [:year, :minute]\n    iex> assert_structs_equal(left, right, keys)\n    true\n\nThe second is to pass a comparison function that returns a boolean that\ndetermines if the structs are equal. When using a comparison function, the\nfirst argument to the function is the `left` struct and the second argument\nis the `right` struct.\n\n    iex> left = DateTime.utc_now()\n    iex> right = DateTime.utc_now()\n    iex> comparison = &(&1.year == &2.year and &1.minute == &2.minute)\n    iex> assert_structs_equal(left, right, comparison)\n    true","ref":"Assertions.html#assert_structs_equal/3"},{"type":"macro","title":"Assertions.refute!/1","doc":"Asserts that the return value of the given expression is `false`.\n\nThis is different than the normal behavior of `refute/1` since that will pass\nif the value is either `false` or `nil`. This is a stricter assertion, only\npassing if the value is `false`. This is very helpful for testing values that\nare expected to only be booleans.\n\nThis will also check specifically for `nil` values when using `>`, `<`, `>=`\nor `<=` since those frequently have unintended behavior.\n\n    iex> refute!(5 > 10)\n    true\n    iex> refute!(\"a\" == \"A\")\n    true","ref":"Assertions.html#refute!/1"},{"type":"type","title":"Assertions.comparison/0","doc":"","ref":"Assertions.html#t:comparison/0"},{"type":"type","title":"Assertions.comparison_or_list/0","doc":"","ref":"Assertions.html#t:comparison_or_list/0"},{"type":"module","title":"Assertions.Absinthe","doc":"Helpful assertions for testing Absinthe applications.\n\nThis module contains some functions that make it much simpler and safer to test Absinthe\napplications. Probably the most common issue that is seen in Absinthe applications is untested\nresolver functions and fields, and it's nearly impossible to tell using just code coverage\nwhich fields are tested or not.\n\nThese functions make it trivially easy to generate very large, comprehensive queries for our\ntypes in Absinthe that will resolve every field in that type (and any number of subtypes as\nwell to a given level of depth), and by default makes it so that we're either testing\nequality of the response or testing a pattern match on the response.\n\nWhile many of these functions explicitly take the schema as the first argument, if you want to\nsimplify things you can use the provided ExUnit case template like so:\n\n  `use Assertions.AbsintheCase, async: true, schema: MyApp.Schema`\n\nand then all functions in this module will not need the schema passed explicitly into it.","ref":"Assertions.Absinthe.html"},{"type":"function","title":"Assertions.Absinthe.assert_response_equals/4","doc":"Assert that the response for sending `document` equals `expected_response`.\n\nThis is helpful when you want to exhaustively test something by asserting equality on every\nfield in the response.","ref":"Assertions.Absinthe.html#assert_response_equals/4"},{"type":"function","title":"Example - Assertions.Absinthe.assert_response_equals/4","doc":"query = \"{ user { #{document_for(:user, 2)} } }\"\n    expected = %{\"user\" => %{\"name\" => \"Bob\", \"posts\" => [%{\"title\" => \"A post\"}]}}\n    assert_response_equals(query, expected)","ref":"Assertions.Absinthe.html#assert_response_equals/4-example"},{"type":"macro","title":"Assertions.Absinthe.assert_response_matches/4","doc":"Assert that the response for sending `document` matches `expr`.\n\nThis is helpful when you want to test some but not all fields in the returned response, or\nwould like to break up your assertions by binding variables in the body of the match and then\nmaking separate assertions further down in your test.","ref":"Assertions.Absinthe.html#assert_response_matches/4"},{"type":"macro","title":"Example - Assertions.Absinthe.assert_response_matches/4","doc":"query = \"{ user { #{document_for(:user, 2)} } }\"\n    assert_response_matches(query) do\n      %{\"user\" => %{\"name\" => \"B\" <> _, \"posts\" => posts}}\n    end\n     assert length(posts) == 1","ref":"Assertions.Absinthe.html#assert_response_matches/4-example"},{"type":"function","title":"Assertions.Absinthe.document_for/4","doc":"Returns a document containing the fields in a type and any sub-types down to a limited depth of\nnesting (default `3`).\n\nThis is helpful for generating a document to use for testing your GraphQL API. This function\nwill always return all fields in the given type, ensuring that there aren't any accidental\nfields with resolver functions that aren't tested in at least some fashion.","ref":"Assertions.Absinthe.html#document_for/4"},{"type":"function","title":"Example - Assertions.Absinthe.document_for/4","doc":"document_for(:user, 2)\n    \"\"\"\n    name\n    age\n    posts {\n      title\n      subtitle\n    }\n    comments {\n      body\n    }\n    \"\"\"","ref":"Assertions.Absinthe.html#document_for/4-example"},{"type":"function","title":"Assertions.Absinthe.fields_for/3","doc":"Returns all fields in a type and any sub-types down to a limited depth of nesting (default `3`).\n\nThis is helpful for converting a struct or map into an expected response that is a bare map\nand which can be used in some of the other assertions below.","ref":"Assertions.Absinthe.html#fields_for/3"},{"type":"module","title":"Assertions.AbsintheCase","doc":"A wrapper for `ExUnit.Case` that provides all assertions in the library and is configured\nespecially for testing Absinthe applications.","ref":"Assertions.AbsintheCase.html"},{"type":"module","title":"Assertions.Case","doc":"A wrapper for `ExUnit.Case` that provides all assertions in the library.","ref":"Assertions.Case.html"},{"type":"module","title":"Assertions.Changeset","doc":"","ref":"Assertions.Changeset.html"},{"type":"extras","title":"Assertions","doc":"# Assertions [![Hex Version](https://img.shields.io/hexpm/v/assertions.svg)](https://hex.pm/packages/assertions) [![Build Status](https://travis-ci.org/devonestes/assertions.svg?branch=master)](https://travis-ci.org/devonestes/assertions)","ref":"readme.html"},{"type":"extras","title":"Installation - Assertions","doc":"Add `assertions` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [{:assertions, \"~> 0.10\", only: :test}]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Assertions","doc":"","ref":"readme.html#usage"},{"type":"extras","title":"Importing - Assertions","doc":"If you only want some assertions in a given module, then import just the\nfunctions that you want to use. Otherwise you can simply call `import\nAssertions` and all assertions will be directly available to your test code.\n\n```elixir\ndef UsersTest do\n  use ExUnit.Case, async: true\n\n  require Assertions\n  import Assertions, only: [assert_lists_equal: 2]\n\n  # ...\nend\n```\n\nBecause these assertions are all macros, `Assertions` must be `require`d first\nif you want to call a function like `Assertions.assert_map_in_list/3`.\n\nImporting assertions in an existing test case (like `MyApp.DataCase` in a\nPhoenix application) is typically recommended.\n\nHere is an example of how you'd add assertions to your `MyApp.DataCase`:\n```elixir\ndefmodule MayApp.DataCase do\n  use ExUnit.CaseTemplate\n\n  using do\n    quote do\n      import Ecto\n      import Ecto.Changeset\n      import Ecto.Query\n\n      import MyApp.DataCase\n\n      # Add the following line\n      import Assertions\n    end\n  end\nend\n```","ref":"readme.html#importing"},{"type":"extras","title":"Assertions.Case - Assertions","doc":"If you want to have all assertions available to you by default, you can use the\nprovided `Assertions.Case` macro. This is a very small wrapper around\n`ExUnit.Case`, and imports all assertions for your use.\n\n```elixir\ndef MyApp.UserTest do\n  use Assertions.Case, async: true\n\n  # ...\nend\n```\n\nBut importing assertions in an existing test case (like `MyApp.DataCase` in a\nPhoenix application) is typically recommended.","ref":"readme.html#assertions-case"},{"type":"extras","title":"Why use `assertions`? - Assertions","doc":"There are three things this library offers:\n\n1) Concise, expressive assertions for common types of tests\n2) Flexibility through composition\n3) Exceptional error messages\n\nLet's look at examples of all of these points. Let's say you have the following\ntest:\n\n```elixir\ndefmodule UsersTest do\n  use ExUnit.Case, async: true\n\n  describe \"update_all/2\" do\n    test \"updates the given users in the database and returns those updated users\" do\n      alice = Factory.insert(:user, name: \"Alice\")\n      bob = Factory.insert(:user, name: \"Bob\")\n\n      updated_names = \n        [{alice, %{name: \"Alice A.\"}, {bob, %{name: \"Bob B.\"}}}]\n        |> Users.update_all()\n        |> Enum.map(& &1.name)\n\n      all_user_names =\n        User\n        |> Repo.all()\n        |> Enum.map(& &1.name)\n\n      Enum.each([\"Alice A.\", \"Bob B.\"], fn name ->\n        assert name in updated_names\n        assert name in all_user_names\n      end\n    end\n  end\nend\n```\n\nTesting elements in lists is a very common thing to do, but it's also very\ntricky! If you want to test those lists, you can't assert that they're equal\nbecause order matters with lists. Also, with those structs, you can't compare\nthem directly because maybe there are associations that might be loaded in one\nstruct but not loaded in the other. The above test is the best you can do to\naccurately test those changes with the standard testing tools.","ref":"readme.html#why-use-assertions"},{"type":"extras","title":"Expressive assertions - Assertions","doc":"But, with `assertions`, you can write that test like this:\n\n```elixir\ndefmodule UsersTest do\n  use ExUnit.Case, async: true\n  import Assertions, only: [assert_lists_equal: 2]\n\n  describe \"update_all/2\" do\n    test \"updates the given users in the database and returns those updated users\" do\n      alice = Factory.insert(:user, name: \"Alice\")\n      bob = Factory.insert(:user, name: \"Bob\")\n\n      result = Users.update_all([{alice, %{name: \"Alice A.\"}, {bob, %{name: \"Bob B.\"}}}\n\n      result\n        |> Enum.map(& &1.name)\n        |> assert_lists_equal([\"Alice A.\", \"Bob B.\"])\n\n      assert_lists_equal(result, Users.list_all(), &assert_structs_equal(&1, &2, [:name]))\n    end\n  end\nend\n```\n\n`assert_lists_equal` asserts that the two lists are equal without taking order\ninto account, which is most often the assertion that we want to make when\ncomparing lists.","ref":"readme.html#expressive-assertions"},{"type":"extras","title":"Flexibility through composition - Assertions","doc":"But `assert_lists_equal` also solves the other problem we had when we wanted to\ncompare lists of structs. That second assertion:\n\n```elixir\nassert_lists_equal(result, Users.list_all(), &assert_structs_equal(&1, &2, [:name]))\n```\n\nis comparing that the two lists are equal, but we give it a custom comparison\nfunction. If we were to just use `assert_lists_equal(results, Users.list_all())`,\nthen all values for all keys in those structs must be equal for them to be\nconsidered equal, and this can be very error prone, especially when dealing with\nstructs used as Ecto resources that can have associations that are either loaded\nor not loaded.\n\nThis ability to compose behavior of assertions lets you easily customize your\nassertions for each of your tests.","ref":"readme.html#flexibility-through-composition"},{"type":"extras","title":"Exceptional error messages - Assertions","doc":"`assertions` always tries to give you the most helpful error messages possible\nfor any test failures to make it easy to see what went wrong and how to fix it.\nLets look at a different test:\n\n```elixir\ntest \"a map in a list matches the value of this other map\" do\n  map = %{key: :value, stores: :are, really: :helpful}\n\n  list = [\n    %{big: :map, with: :lots, of: :keys},\n    %{another: :big, store: :with, key: :values}\n  ]\n\n  assert Enum.any?(list, fn map_in_list ->\n            Map.get(map_in_list, :key) == map.key\n          end)\nend\n```\n\nThe output you get from that failure looks like this:\n\n```\n     Expected truthy, got false\n     code: assert Enum.any?(list, fn map_in_list -> Map.get(map_in_list, :key) == map.key() end)\n     arguments:\n\n         # 1\n         [\n           %{big: :map, of: :keys, with: :lots},\n           %{another: :big, key: :values, store: :with}\n         ]\n\n         # 2\n         #Function<21.25555998/1 in Assertions.FailureExamples.\"test example\"/1>\n\n     stacktrace:\n       test/failure_examples.exs:207: (test)\n```\n\nThat's not really helpful. What we wanted to know was essentially \"is a map with\nthe same values for a certain key in this list?\", and we got no help in finding\nwhat went wrong. With `assertions` we can write that same assertion like this:\n\n```elixir\ntest \"a map in a list matches the value of this other map\" do\n  map = %{key: :value, stores: :are, really: :helpful}\n\n  list = [\n    %{big: :map, with: :lots, of: :keys},\n    %{another: :big, store: :with, key: :values}\n  ]\n\n  assert_map_in_list(map, list, [:key])\nend\n```\n\nAnd then the output looks like this:\n\n```\n     Map matching the values for keys `:key` not found\n     code:  assert_map_in_list(map, list, [:key])\n     arguments:\n\n         # 1\n         %{key: :value, really: :helpful, stores: :are}\n\n         # 2\n         [\n           %{big: :map, of: :keys, with: :lots},\n           %{another: :big, key: :values, store: :with}\n         ]\n\n     left:  %{key: :value}\n     right: [%{}, %{key: :values}]\n     stacktrace:\n       test/failure_examples.exs:206: (test)\n```\n\nThe error message there in the `left` and `right` keys shrinks down the output\nto only show us the relevant information. I can see here that one map didn't\ncontain the key we wanted, and the value was different in the other.","ref":"readme.html#exceptional-error-messages"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,49]}}